package me.yoqi.android.barcode;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.Point;
import android.os.Bundle;
import android.view.Display;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;

import com.google.zxing.Result;
import com.google.zxing.ResultMetadataType;
import com.google.zxing.client.result.ParsedResult;
import com.google.zxing.client.result.ResultParser;

import java.text.DateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.EnumSet;
import java.util.Map;

import me.yoqi.android.zxingscanview.widget.ZxingForegroundView;
import me.yoqi.android.zxingscanview.widget.ZxingScanView;

public class MainActivity extends AppCompatActivity implements ZxingScanView.OnScanListener {

    private static final int PERMISSIONS_REQUEST_CAMERA = 108;
    private static final Collection<ResultMetadataType> DISPLAYABLE_METADATA_TYPES =
            EnumSet.of(ResultMetadataType.ISSUE_NUMBER,
                    ResultMetadataType.SUGGESTED_PRICE,
                    ResultMetadataType.ERROR_CORRECTION_LEVEL,
                    ResultMetadataType.POSSIBLE_COUNTRY);

    private ZxingScanView mVScan;
    private ZxingForegroundView mVForeground;

    public static void getInstance(Context context) {
        context.startActivity(new Intent(context, MainActivity.class));
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //判断屏幕方向
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_FULL_SENSOR);
        if (screenOrient(this) == 0) {
            setContentView(R.layout.activity_main__landscape);
        } else {
            setContentView(R.layout.activity_main);
        }

        Toolbar toolbar = findViewById(R.id.zsv_toolbar);
        toolbar.setVisibility(View.GONE);

        mVScan = findViewById(R.id.zsv_zsv_scan);
        mVForeground = findViewById(R.id.zsv_zfv_foreground);
        mVScan.addOnScanListener(this);
    }

    @Override
    public void onError(ZxingScanView scanView) {
        switch (scanView.getErrorCode()) {
            case ZxingScanView.ERROR_CODE_0:
                // 相机无法打开
                Toast.makeText(this, R.string.zsv_error, Toast.LENGTH_SHORT).show();
                break;
            case ZxingScanView.ERROR_CODE_1:
                // 缺少打开相机的权限
                if (!ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.CAMERA)) {
                    ActivityCompat.requestPermissions(this,
                            new String[]{Manifest.permission.CAMERA},
                            PERMISSIONS_REQUEST_CAMERA);
                }
                break;
        }
    }

    /**
     * 扫描结果监听器
     *
     * @param scanView    ZxingScanView
     * @param result      结果 扫描结果
     * @param barcode     图片 条码图片
     * @param scaleFactor 缩放比
     */
    @Override
    public void onResult(ZxingScanView scanView, Result result, Bitmap barcode, float scaleFactor) {
        ParsedResult parsedResult = ResultParser.parseResult(result);
        final String format = "format:" + result.getBarcodeFormat().toString();
        final String type = "type:" + parsedResult.getType().toString();
        DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
        final String date = "date:" + formatter.format(new Date(result.getTimestamp()));
        String meta = "";
        Map<ResultMetadataType, Object> metadata = result.getResultMetadata();
        if (metadata != null) {
            StringBuilder metadataText = new StringBuilder(20);
            for (Map.Entry<ResultMetadataType, Object> entry : metadata.entrySet()) {
                if (DISPLAYABLE_METADATA_TYPES.contains(entry.getKey())) {
                    metadataText.append(entry.getValue()).append('\n');
                }
            }
            if (metadataText.length() > 0) {
                metadataText.setLength(metadataText.length() - 1);
                meta = metadataText.toString();
            }
        }
        CharSequence displayContents = parsedResult.getDisplayResult();
        Toast.makeText(this, format + "\n" + type + "\n" + date + "\n" + meta + "\n" + displayContents,
                Toast.LENGTH_SHORT).show();
        // 重新扫描
        scanView.restartScanDelay(3000);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == PERMISSIONS_REQUEST_CAMERA) { //获取照相机权限
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                mVScan.open(); // 打开摄像头
            } else {
                mVForeground.setMode(ZxingForegroundView.MODE_ERROR);
            }
        }
    }

    /**
     * 判定当前的屏幕是竖屏还是横屏
     *
     * @param activity 当前activity对象
     * @return orient 0横屏，1竖屏
     */
    public int screenOrient(Activity activity) {
        int orient = activity.getRequestedOrientation();
        //获取屏幕方向，但是如果关闭反转屏幕，则不会反转。但是手机确实翻转了，所以需要手动根据屏幕长宽比判断
        if (orient != ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE && orient != ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) {
            Display display = activity.getWindowManager().getDefaultDisplay();
            Point outSize = new Point();
            display.getSize(outSize);
            int screenWidth = outSize.x; //宽
            int screenHeight = outSize.y; //高
            orient = screenWidth > screenHeight ? ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE : ActivityInfo.SCREEN_ORIENTATION_PORTRAIT; //宽>高：横屏
        }
        return orient;
    }
}